"""
Author: chong.ma
Date: 2023-11-15 15:48:34
LastEditTime: 2024-01-19 20:31:27
FilePath: /root/bench_tool/bench_tool/pitbench.py
Description:

"""

import argparse
import os
import re
import sys
from pathlib import Path

from utils.BenchManagers import BenchCheck, BenchReset
from utils.ConfigManagers import DeviceManager
from utils.Utils import Utils

sys.path.append(os.path.dirname(os.path.dirname(__file__)))
BENCH_TOOL_DIR = Path(__file__).resolve().parents[0]

USAGE = """
1. 执行台架上的CHECK/RESET操作
2. 开关唤醒帧
3. 关于开关参数：按bit位顺序，从左到右，0表示不执行，1表示执行 # 741 -> 111 100 001
第一组 111
在RESET操作下:重置vlan          重置can        重置唤醒报文
在CHECK操作下:检查dut联通性     检查can消息    检查组播报文
第二组 111
在RESET操作下:上下电            重置电阻板卡    重置网络唤醒
在CHECK操作下:检查串口联通性    检查NI板卡      检查用户状态
第三组 111
在RESET操作下:暂无
在CHECK操作下:检查dut版本

python pitbench.py --action CHECK --switch 740
python pitbench.py --action RESET --switch 740
python pitbench.py --wake_up ON
python pitbench.py --wake_up OFF
"""
def parse_args():
    parser = argparse.ArgumentParser(description="Perform operations on the pitbench", usage=USAGE)
    parser.add_argument("--action", choices=['CHECK', 'RESET'], help="Specify 'CHECK' or 'RESET' operation")
    parser.add_argument("--wake_up", choices=['ON', 'OFF'], help="Turn ON or OFF the wake-up frame")
    parser.add_argument("--switch", type=str, default="700", help="Binary switch settings for CHECK/RESET operations, e.g., 740")
    return parser.parse_args()

def main():
    args = parse_args()
    dm = DeviceManager()

    if args.action:
        process_action(args, dm)
    
    if args.wake_up:
        process_wake_up(args.wake_up, dm)

def process_action(args, dm):
    operation = BenchCheck() if args.action == "CHECK" else BenchReset()
    switch_settings = [bit == '1' for bit in f"{int(args.switch):09b}"]

    with operation as op:
        if args.action == "CHECK":
            execute_check(op, dm, switch_settings)
        elif args.action == "RESET":
            execute_reset(op, dm, switch_settings)

def execute_check(checker, dm, switch):
    vlans = dm.BENCH_DICT.get("vlans", {})
    # 尝试匹配名为 VLAN11 或包含 11 的 VLAN
    vlan11_iface = next((vlan for vlan in vlans if re.match(r".*11.*", vlan)), None)  
    if not vlan11_iface:
        print("Error: VLAN 11 not found. Skipping multicast check.")
    else:
        print(f"VLAN 11 interface found: {vlan11_iface}")
    
    # 第一组操作
    if switch[0]:  
        checker.check_net_connect(dm.BENCH_DICT["ip_addresses"])
    if switch[1]:  # 检查CAN消息
        checker.check_can_message(dm.BENCH_DICT["can"])
    if switch[2]:  # 检查组播报文
        checker.check_multicast(dm.BENCH_DICT["ip_addresses"], vlan11_iface)  # iface需具体指定

    # 第二组操作
    if switch[3]:  # 检查串口联通性
        checker.check_serial(dm.BENCH_DICT["serial_type"], dm.BENCH_DICT["serials"])
    if switch[4]:  # 检查NI板卡
        checker.check_daq()
    if switch[5]:  # 检查用户状态
        checker.check_user_status()

    # 第三组操作
    if switch[6]:  # 检查DUT版本
        checker.check_version(dm.BENCH_DICT["uds"])

def execute_reset(reseter, dm, switch):
    
    if switch[0]:  # 重置VLAN
        reseter.execute_reset_vlan(dm.BENCH_DICT["vlans"])
    if switch[1]:  # 重置CAN
        reseter.execute_reset_can(dm.BENCH_DICT["can_channel_range"][0], dm.BENCH_DICT["can_channel_range"][1])
    if switch[2]:  # 发送唤醒报文
        reseter.execute_wake_up(dm.BENCH_DICT["wake_up_frame"])

    # 第二组操作
    if switch[3]:  # 上下电
        reseter.execute_reset_power_by_relay(dm.BENCH_DICT["relay_type"], dm.BENCH_DICT["dut_type"])
        reseter.execute_reset_power(dm.BENCH_DICT["power_type"], dm.BENCH_DICT["powers"])
    if switch[4]:  # 重置电阻板卡
        reseter.execute_reset_res_box(dm.BENCH_DICT["can"]["Resistor"], dm.BENCH_DICT["res_box_ids"])
    if switch[5]:  # 网络唤醒
        reseter.execute_net_wake_up(dm.BENCH_DICT["net_wake_up_frame"])

def process_wake_up(wake_up_state, dm):
    frames = dm.BENCH_DICT.get("wake_up_frame")
    with BenchReset() as reseter:
        if wake_up_state == "ON":
            reseter.execute_wake_up(frames)
        else:
            reseter.execute_wake_up(frames, only_stop=True)

if __name__ == "__main__":
    main()