"""
Author: chong.ma
Date: 2024-03-28 15:56:27
LastEditTime: 2024-04-09 19:36:40
FilePath: /root/bench_tool/bench_tool/usb-relay.py
Description:

"""

import argparse
import re
import time

from utils.ConfigManagers import DeviceManager
from utils.RelayManagers import RelayManager

USAGE = """
usb继电器板，智嵌物联的串口继电器板
1. 检查DI状态，设置DO输出，对单独的通道进行操作
2. 上下电操作，对测试板子进行上下电，需要在config里添加relays配置
3. EFUSE操作，对EFUSE通道进行三态选择操作（通过双继电器进行），需要在config里添加relays配置
4. MODE操作，对CCC的MODE进行切换，需要在config里添加relays配置

python3 usb-relay.py --type DI --src /dev/usb_relay@01
python3 usb-relay.py --type DO --src /dev/usb_relay@01 --value ON
python3 usb-relay.py --power CCC --value ON
python3 usb-relay.py --power F_ZONE --value OFF
python3 usb-relay.py --power R_ZONE --value OFF
python3 usb-relay.py --power L_ZONE --value OFF
python3 usb-relay.py --efuse CH01 --state LOAD
python3 usb-relay.py --efuse CH01 --state GND
python3 usb-relay.py --efuse CH01 --state NULL
python3 usb-relay.py --mode SERIAL
python3 usb-relay.py --mode RCON
"""

if __name__ == "__main__":
    dm = DeviceManager()
    parser = argparse.ArgumentParser(usage=USAGE)

    def validate_arg_format(value):
        if not re.match(r"^CH[0-9][0-9]$", value):
            raise argparse.ArgumentTypeError("arg2 必须以 'CHXX' 格式")
        return value

    parser.add_argument("--type", type=str, required=False, help="eg. DI/DO")
    parser.add_argument("--src", type=str, required=False, help="eg. /dev/usb_relay&01")
    parser.add_argument("--value", type=str, required=False, help="eg. ON/OFF/RESET")
    parser.add_argument("--mode", type=str, required=False, help="eg. RCON/SERIAL")
    parser.add_argument("--power", type=str, required=False, help="eg. CCC/F_ZONE/R_ZONE/L_ZONE")
    parser.add_argument("--efuse", type=validate_arg_format, required=False, help="eg. CH01")
    parser.add_argument("--state", type=str, required=False, help="eg. LOAD/GND/NULL")

    args = parser.parse_args()

    if dm.BENCH_DICT.get("relay_type") == "usb-relay":
        relay_dict = dm.BENCH_DICT.get("relays")

        if args.mode:  # 处理MODE操作
            ser_port, addr, channel, dir = relay_dict.get("CCC_Power_Btn")
            with RelayManager("CCC_Power_Btn", ser_port, addr, channel, dir) as relay:
                relay.set_relay_single("Off")
            time.sleep(3)
            act = "Off" if args.mode == "SERIAL" else "On"
            for button in ["CCC_Mode_Btn_1", "CCC_Mode_Btn_2", "CCC_Mode_Btn_3", "CCC_Mode_Btn_4"]:
                ser_port, addr, channel, dir = relay_dict.get(button)
                with RelayManager(button, ser_port, addr, channel, dir) as relay:
                    relay.set_relay_single(act)
            time.sleep(3)
            ser_port, addr, channel, dir = relay_dict.get("CCC_Power_Btn")
            with RelayManager("CCC_Power_Btn", ser_port, addr, channel, dir) as relay:
                relay.set_relay_single("On")

        elif args.power:  # 处理上下电操作
            button = f"{args.power}_Power_Btn"
            assert relay_dict.get(button) is not None, f"button {button} not found"
            ser_port, addr, channel, dir = relay_dict.get(button)
            act = "Off" if args.value == "OFF" else ("On" if args.value == "ON" else "Reset")
            with RelayManager(button, ser_port, addr, channel, dir) as relay:
                relay.set_relay_single(act)

        elif args.efuse:  # 处理EFUSE操作
            ser_port, addr, channel, dir = relay_dict.get(f"EFUSE_{args.efuse}")
            with RelayManager(f"EFUSE_{args.efuse}", ser_port, addr, channel, dir) as relay:
                relay.set_relay_double(args.state)

        elif args.type == "DI":  # 处理DI/DO操作
            src = args.src.split("@")
            with RelayManager(args.src, src[0], "000000", src[1], "input") as relay:
                relay.read_input()
        elif args.type == "DO":
            src = args.src.split("@")
            act = "Off" if args.value == "OFF" else ("On" if args.value == "ON" else "Reset")
            with RelayManager(args.src, src[0], "000000", src[1], "output") as relay:
                relay.set_relay_single(act)

        else:
            print("Invalid arg1")

    # with RelayManager("Ecall_Led_Red") as relay_in:
    #     relay_in.read_input()
