# flake8: noqa: D202 E501
"""
Author: chong.ma
Date: 2023-11-09 20:43:11
LastEditTime: 2024-01-19 20:36:06
FilePath: /root/bench_tool/bench_tool/utils/BenchManagers.py
Description: 用于管理台架的类

"""

import ast
import concurrent.futures
import multiprocessing
import os
import re
import subprocess
import sys
import time
from pathlib import Path
from pickle import TRUE
from socket import (
    AF_INET,
    IP_MULTICAST_TTL,
    IPPROTO_IP,
    IPPROTO_UDP,
    SOCK_DGRAM,
    socket,
)

from scapy.all import IP, UDP, Ether, sniff

from utils.IOManagers import NI_Managers
from utils.RelayManagers import RelayBoardDriver
from utils.ResManagers import ResManager
from utils.SerialManagers import NET_Serials, USB_Serials
from utils.Utils import Utils

from .Logger import get_logger

sys.path.append(os.path.dirname(os.path.dirname(__file__)))
BENCH_TOOL_DIR = Path(__file__).resolve().parents[1]
logger = get_logger("BenchManagers")


class BenchCheck:
    def __init__(self):
        self.failed_flag = False

    def __enter__(self, **kwargs):
        print("Entering the context")
        return self  # 返回一个用于上下文的对象

    def __exit__(self, exc_type, exc_value, traceback):
        self.print_result()
        print("Exiting the context")

    def print_result(self):
        if self.failed_flag:
            print("Check failed")
        else:
            print("Check success")

    def check_user_status(self):
        print("Checking user status...")
        cmd = f'bash -c "{BENCH_TOOL_DIR}/scripts/check_state.sh"'
        print(subprocess.check_output(cmd, shell=True).decode("utf-8"))

    def check_version(self, uds_dict):
        print("Checking version...")
        for label, uds in uds_dict.items():
            addr = uds.get("logical_addr")
            cmd = f'bash -c "rop-cli uds readdid -i 172.20.10.1 -e {addr} -d f118"'
            res = subprocess.check_output(cmd, shell=True).decode("utf-8").strip("\n")
            res = res if (res != "" and len(res) < 20) else "Failed"
            print(f"{label.ljust(10)}\t--> {addr.ljust(30)}: {res}")
        #   cmd = f"echo "
        # cmd = f"echo 'version' | nc -w 2 -u -p 52123 172.20.10.23 53053"

    def check_net_connect(self, ip_dict):
        print("Checking NET connection...")
        with concurrent.futures.ThreadPoolExecutor() as executor:
            # 使用线程池并行执行 check_single_ip 函数
            futures = {executor.submit(self.__check_single_ip, ip, label): (ip, label) for ip, label in ip_dict.items()}
            for future in concurrent.futures.as_completed(futures):
                ip, label = futures[future]
                try:
                    if future.result() == 0:
                        print(f"{ip.ljust(10)}\t--{label.ljust(30)}: Success")
                    else:
                        print(f"{ip.ljust(10)}\t--{label.ljust(30)}: Failed")
                        self.failed_flag = True
                except Exception as e:
                    print(f"Error checking {ip}: {e}")
                    self.failed_flag = True

    def __check_single_ip(self, ip, label):
        try:
            result = subprocess.run(["ping", "-c", "1", ip], stdout=subprocess.PIPE, stderr=subprocess.PIPE)
            return result.returncode
        except Exception as e:
            print(f"Error checking {ip}: {e}")
            return 1  # 假设出现异常时将返回码设置为非零

    def check_multicast(self, ip_dict, iface="vlan.11"):
        print("Checking MUTICAST message...")
        with concurrent.futures.ThreadPoolExecutor() as executor:
            # 使用线程池并行执行 check_single_multicast 函数
            futures = {executor.submit(self.__check_single_multicast, ip, label, iface): (ip, label) for ip, label in ip_dict.items() if ip.startswith("172.20.11")}
            for future in concurrent.futures.as_completed(futures):
                ip, label = futures[future]
                try:
                    if future.result() is True:
                        print(f"{ip.ljust(10)}\t--{label.ljust(30)}: found")
                    else:
                        print(f"{ip.ljust(10)}\t--{label.ljust(30)}: not found")
                        self.failed_flag = True
                except Exception as e:
                    print(f"Error checking {ip}: {e}")
                    self.failed_flag = True

    def __check_single_multicast(self, ip, label, iface="vlan.11"):
        try:
            captured_packet = sniff(iface=iface, count=3, filter=f"src host {ip} and ip dst 224.1.1.11", timeout=5)
            # 验证是否有 IP 地址为 172.20.11.6 的设备向 224.1.1.11 发送组播报文
            if len(captured_packet) > 0:
                for packet in captured_packet:
                    if UDP in packet and packet[IP].src == ip and packet[0][IP].dst == "224.1.1.11":
                        return True
                return False
            else:
                return False

        except Exception as e:
            print(f"Error checking {ip}: {e}")
            return 1  # 假设出现异常时将返回码设置为非零

    def check_can_message(self, can_dict):
        print("Checking CAN message...")
        # 运行 ip link 命令来获取所有网络接口
        result = subprocess.run(["ip", "link"], stdout=subprocess.PIPE, text=True, check=True)
        # 在输出中查找以 "can" 开头的接口名称
        can_interfaces = [iface for iface in re.findall(r"(\w+):\s+<", result.stdout) if iface.startswith("can")]
        with concurrent.futures.ThreadPoolExecutor() as executor:
            # 使用线程池并行执行 check_single_can 函数
            futures = {executor.submit(self.__check_single_can, label, can, can_interfaces): (label, can) for label, can in can_dict.items()}
            for future in concurrent.futures.as_completed(futures):
                label, can = futures[future]
                try:
                    if future.result() is None:
                        print(f"{can.ljust(15)} -- {label.ljust(29)}: Not found")
                    elif future.result():
                        print(f"{can.ljust(15)} -- {label.ljust(29)}: Success")
                    else:
                        print(f"{can.ljust(15)} -- {label.ljust(29)}: Failed")
                        self.failed_flag = True
                except Exception as e:
                    print(f"Error checking {can}: {e}")
                    self.failed_flag = True

    def __check_single_can(self, label, can, can_interfaces):
        try:
            if can.lower() in can_interfaces:
                cmd = f"timeout -s TERM 2 candump {can.lower()} | head -n 1"
                result = subprocess.run(cmd, shell=True, stdout=subprocess.PIPE)
                return result.stdout != b""
            else:
                return None
        except Exception as e:
            print(f"Error checking {can}: {e}")
            return False

    def check_can_devices(self):
        # grep PEAK_ /boot/config-`uname -r`
        # lsmod | grep ^peak
        pass

    def check_daq(self):
        print("Checking DAQ...")
        with NI_Managers() as ni:
            ni.find_test_devices()
        # self.ping_and_print_result()

    def check_serial(self, serial_type, serial_dict):
        print("Checking serial...")
        if serial_type == "net-serial":
            with NET_Serials(serial_dict) as serial:
                serial.check_serial()
        elif serial_type == "usb-serial":
            with USB_Serials(serial_dict) as serial:
                pass


class BenchReset:
    def __init__(self):
        self.socket = None
        pass

    def __enter__(self, **kwargs):
        print("Entering the context")
        return self  # 返回一个用于上下文的对象

    def __exit__(self, exc_type, exc_value, traceback):
        if self.socket is not None:
            self.socket.close()
        print("Exiting the context")

    def execute_net_wake_up(self, frames: dict):
        print("Executing NET wake up...")
        self.socket = socket(AF_INET, SOCK_DGRAM, IPPROTO_UDP)
        self.socket.setsockopt(IPPROTO_IP, IP_MULTICAST_TTL, 20)
        # self.socket.setsockopt(self.socket.SOL_IP, self.socket.IP_MULTICAST_IF, self.socket.inet_aton("172.20.10.250"))
        self.socket.bind(("172.20.10.123", 60000))
        self.socket.settimeout(1.0)
        host_ip = ("239.192.255.255", 60000)
        # host_ip = ("172.20.10.24", 60000)
        data_bas = [bytearray(ast.literal_eval(frame)) for frame in frames.values()]

        try:
            while TRUE:
                for data_ba in data_bas:
                    self.socket.sendto(data_ba, host_ip)
                    time.sleep(0.1)
        except KeyboardInterrupt:
            print("Exiting...")
            self.socket.close()

    def execute_wake_up(self, can_frames, only_stop=False):
        subprocess.run("killall cangen", shell=True, check=False)
        subprocess.run("crontab -r", shell=True, check=False)
        if not only_stop:
            for frame in can_frames:
                can_channel = frame.get("channel")
                can_id = frame.get("id")
                can_data = frame.get("data")
                self.__execute_single_wake_up(can_channel, can_id, can_data)
                cmd = f"(crontab -l ; echo '*/1 * * * * /bin/bash {BENCH_TOOL_DIR}/scripts/wake_up.sh {can_channel} {can_id} {can_data}') | crontab -"
                subprocess.run(cmd, shell=True, check=False)
            print(subprocess.check_output("crontab -l", shell=True).decode("utf-8"))
            print(subprocess.check_output("ps -aux | grep cangen", shell=True).decode("utf-8"))

    def __execute_single_wake_up(self, can, id, data):
        print("Executing CAN wake up...")
        cmd = ["bash", f"{BENCH_TOOL_DIR}/scripts/wake_up.sh", can, id, data]
        Utils.execute_command(cmd)
        # self.ping_and_print_result()

    def execute_reset_power_by_relay(self, relay_type, dut_type: dict = None):
        print("Executing RELAY reset...")
        if relay_type == "net-relay":
            Utils.execute_py_command(f"{BENCH_TOOL_DIR}/net-relay.py", "ALL", "RESET")
        elif relay_type == "usb-relay":
            for dut in ["ccc", "f_zone", "r_zone", "l_zone"]:
                if dut in dut_type.keys():
                    Utils.execute_py_command(f"{BENCH_TOOL_DIR}/usb-relay.py", "--power", dut.upper(), "--value", "OFF")
            time.sleep(3)
            for dut in ["ccc", "f_zone", "r_zone", "l_zone"]:
                if dut in dut_type.keys():
                    Utils.execute_py_command(f"{BENCH_TOOL_DIR}/usb-relay.py", "--power", dut.upper(), "--value", "ON")

    def execute_reset_power(self, power_type, power_dict: dict = None):
        print("Executing POWER reset...")
        if power_type == "program-power":
            for power in power_dict.keys():
                Utils.execute_py_command(f"{BENCH_TOOL_DIR}/program_power.py", "--whichone", power, "--action", "OFF")
            time.sleep(3)
            for power in power_dict.keys():
                Utils.execute_py_command(f"{BENCH_TOOL_DIR}/program_power.py", "--whichone", power, "--action", "ON")

    def execute_reset_can(self, start, end):
        print("Executing CAN reset...")
        cmd = ["bash", f"{BENCH_TOOL_DIR}/scripts/can_up.sh", start, end]
        Utils.execute_command(cmd)

    def execute_reset_vlan(self, vlans):
        print("Executing VLAN reset...")
        ext = vlans.get("external")
        base = vlans.get("base")
        cmd = ["bash", f"{BENCH_TOOL_DIR}/scripts/reset_net.sh", base.get("ip"), base.get("mac"), ext.get("ip"), ext.get("mac")]
        Utils.execute_command(cmd)

        for key, val in vlans.items():
            if key.startswith("vlan"):
                cmd = ["bash", f"{BENCH_TOOL_DIR}/scripts/add_vlan.sh", key, val.get("id"), val.get("ip"), val.get("mac")]
                Utils.execute_command(cmd)
            elif key.startswith("rc-nif"):
                cmd = ["bash", f"{BENCH_TOOL_DIR}/scripts/add_rc_nif.sh"]
                Utils.execute_command(cmd)
        print(subprocess.check_output("ip addr show", shell=True).decode("utf-8"))

    def execute_reset_res_box(self, res_can, res_boxs):
        print("Executing RES reset...")
        if res_boxs != []:
            with ResManager(channel=res_can) as manager:
                for res_box in res_boxs:
                    for ch in range(1, 17):
                        manager.single_res_cmd(res_box, ch, 2000, "Off")

    def execute_reset_arp(self, arps):
        print("Executing ARP reset...")
        if arps is not None:
            for ip, mac in arps.items():
                subprocess.run(f"arp -d {ip}", shell=True, check=False)
                subprocess.run(f"arp -s {ip} {mac}", shell=True, check=False)
            print(subprocess.check_output("arp -a | grep PERM", shell=True).decode("utf-8"))
