"""
Author: chong.ma
Date: 2023-11-09 21:06:01
LastEditTime: 2023-11-24 14:53:20
FilePath: /root/bench_tool/utils/ConfigManagers.py
Description: 用于管理台架配置文件的类

"""
import json
import os
import re
import subprocess
import sys
from pathlib import Path
from typing import List, Optional, Tuple

import xlrd

sys.path.append(os.path.dirname(os.path.dirname(__file__)))
from .Logger import get_logger
from .Utils import Utils

# (
#     get_config_path_by_ip,
#     get_bench_config_file,
#     get_benchname,
#     get_idls,
#     get_nuc_ip,
#     set_benchname,
#     update_bench_config,
#     update_bench_config_by_dict,
#     update_bench_config_by_key_val,
# )

BENCH_TOOL_DIR = Path(__file__).resolve().parents[1]
BENCH_MAP_PATH = BENCH_TOOL_DIR / "bench_config/bench_map.json"
BASE_DIR = BENCH_TOOL_DIR / "bench_config"
logger = get_logger("ConfigManagers")

RED = "\033[1;31m"
GREEN = "\033[1;32m"
YELLOW = "\033[1;33m"
BLUE = "\033[1;34m"
RESET = "\033[0m"


class ConfigGenerate:
    """用于从excel表格中生成配置文件"""

    def __init__(self, filepath) -> None:
        self.data = xlrd.open_workbook(filepath)
        self.table = self.data.sheet_by_name("Platform")
        self.cartype = "Alps" if "Alps" in filepath else "Leo"

        self.io_pick_list = [
            "DUT",
            "Connector",
            "PIN Number",
            "Circuit Properties",
            "Test HW Itf Type",
            "Connect to - channel",
            "parameters",
            "io_type",
        ]

    def __enter__(self):
        print("entering...")
        self.col_index_connected = self.get_col_index("Connect to - channel")
        dict = self.make_dict()
        self.write_json(f"{self.cartype}.json", dict)
        # print(dict)
        # writ_dict = get_dict(table, pick_list)
        # # data = {"name": "John Doe", "age": 30, "address": {"123 Main St": "Anytown USA"}}
        # write_json("Alps.json", writ_dict)
        # return self

    def __exit__(self, exc_type, exc_val, exc_tb):
        print("exiting...")

    def get_col_index(self, col_name):  # 获取列的索引
        """Get the index of the column in the table."""
        for col in range(self.table.ncols):
            if self.table.cell_value(0, col) == col_name:  # 用第一行的数据去索引，第0行无数据
                return col
        raise ValueError(f"NO SUCH COLUMN: {col_name}")

    def get_can_lin(self):
        dict_can = {}
        dict_lin = {}
        for row in range(1, self.table.nrows):
            if "CAN" in self.table.cell_value(row, self.col_index_connected):
                col_index = self.get_col_index("PIN Definition")
                can_name = self.table.cell_value(row, col_index).split(" - ")[0].replace("CANL", "CAN").replace("CANH", "CAN").replace(" ", "_")
                can_num = self.table.cell_value(row, self.col_index_connected)
                if "LIN" in can_name:
                    dict_lin[can_name] = can_num
                else:
                    dict_can[can_name] = can_num

        return dict_can, dict_lin

    def get_io(self):
        io_list = []
        io_dict = {}
        for row in range(1, self.table.nrows):
            cell_val = self.table.cell_value(row, self.col_index_connected)
            if cell_val != "" and "CAN" not in cell_val:
                for col_name in self.io_pick_list:
                    col_index = self.get_col_index(col_name)
                    cell_val = self.table.cell_value(row, col_index)
                    if type(cell_val) is float:
                        cell_val = str(int(cell_val))
                    io_list.append(cell_val)

        io_lists = [io_list[i : i + 8] for i in range(0, len(io_list), 8)]

        for list in io_lists:
            list = tuple(list)
            try:
                io_dict["".join(list[0] + "." + list[1] + "." + list[2])] = {
                    "pin": list[1] + "-" + list[2],
                    "type": list[4],
                    "io_type": list[7],
                    "phy_chans": list[5],
                    "KWARGS": json.loads(list[6]) if list[6] != "" else {},
                }
            except:
                raise ValueError(f"KWARGS ERROR {list[6]}")

        return io_dict

    def make_dict(self):
        final_dict = {}
        can, lin = self.get_can_lin()
        io = self.get_io()
        final_dict["CAN"] = can
        final_dict["LIN"] = lin
        final_dict["IO"] = io

        return final_dict

    def write_json(self, file, dict_to_write):
        """Write the generated configuration to the specified file."""
        with open(file, "w+", encoding="utf-8") as f:
            json.dump(dict_to_write, f, indent=2, sort_keys=False, ensure_ascii=False)


class CanConfig:
    """@Description : Porting from Alpha2/utility/, used for generating udev rules file based on the current system configuration.
    @Examples    : python3 generate_udev_rule.py
    """

    def __init__(self):
        pass

    def __enter__(self, **kwargs):
        print("Entering the context")
        return self  # 返回一个用于上下文的对象

    def __exit__(self, exc_type, exc_value, traceback):
        print("Exiting the context")

    def __get_sys_class_net_links(self) -> list:
        """Get links from `ls -l /sys/class/net`.

        Perform a system call to get the links from `/sys/class/net` and clean that
        output before performing further processing. Specifically, we're removing
        anything that doesn't start with an "l", because `ls -l` denotes links with
        a leading "l".

        Returns:
            List of strings representing links from `ls -l /sys/class/net`.
        """
        cmd = "ls -l /sys/class/net"
        out = subprocess.check_output(cmd, shell=True)
        out = out.decode("utf-8")
        out = out.split("\n")
        # links, what we're interested in, begin with "l"
        out = [i for i in out if i[:1] == "l"]
        return out

    def __parse_can_interface_links_pattern1(self, data: list = None) -> dict:
        """Parse output from `ls -l /sys/class net`.

        If no arguments are provided, then we perform the default operation, and
        use the output of `ls -l /sys/class/net` to create a mapping from can
        network interface identifiers to a string that will be used in the udev
        persistent net rules to assign a usb device to a can network interface
        identifier. This implicitly assumes that your system is set up correctly in
        the first place.

        If `data` is provided, then that is used instead of `ls -l /sys/class/net`.

        Args:
            data (optional): List of strings representing `ls -l` output

        Returns:
            mapping: A dictionary that maps a can network interface identifer,
                to a string used to identify a usb device using the udev system.
        """
        if data is None:
            data = self.__get_sys_class_net_links()
        can_pattern = re.compile(r"\s(can\d{1,2})\s->\s")
        # match ../../devices/*/*/usb#/
        pth = r"\s\.\.\/\.\.\/devices\/[a-zA-Z0-9:\.]*\/[a-zA-Z0-9:\.]*\/[a-zA-Z0-9:\.]*\/usb\d\/"
        # match previous plus d-d/d-d.d/d-d.d.d/
        pth += r"\d-\d\/\d-\d\.\d\/"
        # these two pieces match a 3 digit path, and a 4 digit path
        dx3 = r"(\d-\d\.\d):\d\.\d\/net"
        dx4 = r"\d-\d\.\d\.\d\/(\d-\d\.\d\.\d):\d\.\d\/net"
        # this matches either the 3 digit path or the 4 digit path
        # using a non-capturing (?:3X|4X) OR operator
        pth += r"(?:" + dx3 + r"|" + dx4 + r")"
        link_pattern = re.compile(pth)
        mapping = {}
        for item in data:
            regex = can_pattern.search(item)
            if not regex:
                continue
            can_interface = regex.group(1)
            regex = link_pattern.search(item)
            if not regex:
                continue
            can_link = regex.group(1) if regex.group(1) else regex.group(2)
            mapping[can_interface] = can_link
        return mapping

    def __parse_can_interface_links_pattern2(self, data: list = None) -> dict:
        """Parse output from `ls -l /sys/class net`.

        If no arguments are provided, then we perform the default operation, and
        use the output of `ls -l /sys/class/net` to create a mapping from can
        network interface identifiers to a string that will be used in the udev
        persistent net rules to assign a usb device to a can network interface
        identifier. This implicitly assumes that your system is set up correctly in
        the first place.

        If `data` is provided, then that is used instead of `ls -l /sys/class/net`.

        Args:
            data (optional): List of strings representing `ls -l` output

        Returns:
            mapping: A dictionary that maps a can network interface identifer,
                to a string used to identify a usb device using the udev system.
        """
        if data is None:
            data = self.__get_sys_class_net_links()
        can_pattern = re.compile(r"\s(can\d{1,2})\s->\s")
        # match ../../devices/*/*/usb#/
        pth = r"\s\.\.\/\.\.\/devices\/[a-zA-Z0-9:\.]*\/[a-zA-Z0-9:\.]*\/usb\d\/"
        # match previous plus d-d/d-d.d/d-d.d.d/
        pth += r"\d-\d\/\d-\d\.\d\/\d-\d\.\d\.\d\/"
        # these two pieces match a 3 digit path, and a 4 digit path
        dx3 = r"(\d-\d\.\d\.\d):\d\.\d\/net"
        dx4 = r"\d-\d\.\d\.\d\.\d\/(\d-\d\.\d\.\d\.\d):\d\.\d\/net"
        # this matches either the 3 digit path or the 4 digit path
        # using a non-capturing (?:3X|4X) OR operator
        pth += r"(?:" + dx3 + r"|" + dx4 + r")"
        link_pattern = re.compile(pth)
        mapping = {}
        for item in data:
            regex = can_pattern.search(item)
            if not regex:
                continue
            can_interface = regex.group(1)
            regex = link_pattern.search(item)
            if not regex:
                continue
            can_link = regex.group(1) if regex.group(1) else regex.group(2)
            mapping[can_interface] = can_link
        return mapping

    def generate_udev_rule(self, filename: str = "70-persistent-net.rules", mapping: dict = {}, prefix: str = "can00") -> list:
        """Generate a udev rule defined by the current setup.

        Using the default arguments, this implicitly assumes that your system is
        set up correctly, meaning that the PCAN-USB devices have been connected to
        the USB strip in the correct order, the NUC has been restarted, the Lion
        board has been reset, the tests have run, and they are all passing. Once
        these things have happened, then the system is in a state where we can
        scan the symbolic links in `/sys/class/net` and write a udev rule to
        maintain that configuration.

        The default operations is to write a file named `70-persistent-net.rule`,
        using data from the command `ls -l /sys/class/net`, but a different filename
        and mapping may be supplied.

        Once finished, you want to move the file to `/etc/udev/rules.d/`.

        Args:
            filename (optional): Filename for udev rule.
            mapping (optional): Mapping used to create udev rule.

        Returns:
            data: List of strings defining a udev rule for network interfaces.
        """
        if not bool(mapping):
            mapping = self.__parse_can_interface_links_pattern1()
        if not bool(mapping):
            mapping = self.__parse_can_interface_links_pattern2()
        data = []
        for can, link in mapping.items():
            # even numbered can network interface identifiers are 0x0, odd are 0x1
            dev_id = int(can[3:]) % 2
            if prefix == "can00":
                tmp = 'KERNELS=="{}", SUBSYSTEM=="net", SUBSYSTEMS=="usb", ATTR{{dev_id}}=="0x{}", NAME="can{:03d}"'
            elif prefix == "can":
                tmp = 'KERNELS=="{}", SUBSYSTEM=="net", SUBSYSTEMS=="usb", ATTR{{dev_id}}=="0x{}", NAME="can{}"'
            data.append(tmp.format(link, dev_id, int(can[3:])))
        data = sorted(data)
        if filename is not None:
            with open(filename, "w") as h:
                h.write("# place in /etc/udev/rules.d/\n")
                for line in data:
                    h.write(line + "\n")
        return data


class DeviceManager(object):
    _instance = None

    def __new__(cls, *args, **kwargs):
        if not cls._instance:
            cls._instance = super(DeviceManager, cls).__new__(cls)
        return cls._instance

    def __init__(self, bench_ip=None):
        """
        Init DeviceManager

        Args:
            bench_ip (str): 指定测试的环境的ip地址
        """
        self.bench_ip = bench_ip if bench_ip else Utils.get_nuc_ip()
        self.BENCH_DICT = Utils.get_bench_config_file(self.bench_ip)
        self.__update_key_for_upper()
        self.update_config_by_envParam(self.bench_ip)

    def __update_key_for_upper(self):
        """Update all can and lin'keys in the self.BENCH_DICT to be upper."""
        new_dict = {}
        if self.get_can_configs():
            for key, value in self.get_can_configs().items():
                new_dict[key.upper()] = value
            self.BENCH_DICT.update({"CAN": new_dict})
        new_dict = {}
        if self.get_lin_configs():
            for key, value in self.get_lin_configs().items():
                new_dict[key.upper()] = value
            self.BENCH_DICT.update({"LIN": new_dict})

    def get_bench_config(self):
        """Get whole bench config by base.json and special json file of bench name

        :return: dict ,whole bench config info .
        """
        return self.BENCH_DICT

    def get_bench_type(self):
        """Get bench type

        :return: str , bench type MONIK | SELF | COMM .
        """
        return self.BENCH_DICT.get("BENCH_TYPE")

    def get_bench_ip(self):
        """
        Get bench ip.

        Return:
            bench ip (str)
        """
        return self.bench_ip

    def set_bench_name(self, ip: str, name: str):
        """
        Set the name of the specified bench env

        Args:
            ip (str): bench'ip
            name (str): bench'name
        """
        return Utils.set_benchname(ip, name)

    def get_bench_name(self):
        """
        Get the name of the specified bench by ATP_RUNNER_BENCH_TYPE or bench map

        Args:
            bench_ip (str): bench'ip,automatically get ip while none
        Return:
            name: bench'name
        """

        try:
            return Utils.get_benchname()
        except:
            with open(BENCH_MAP_PATH) as f:
                bench_map = json.load(f)
                return bench_map.get(self.bench_ip)

    def update_config_by_envParam(self, ip: str):
        """update bench can/lin config by env params, such as ZONE_FL_CAN1

        Args:
            ip (str): bench'ip
        """
        new_dict = {}
        if self.get_can_configs():
            for key in self.get_can_configs().keys():
                env_params = os.getenv(key)
                if env_params is not None:
                    new_dict[key] = env_params
            self.BENCH_DICT.update({"CAN": new_dict})

        new_dict = {}
        if self.get_lin_configs():
            for key in self.get_lin_configs().keys():
                env_params = os.getenv(key)
                if env_params is not None:
                    new_dict[key] = env_params
            self.BENCH_DICT.update({"LIN": new_dict})

        self.__update_key_for_upper()

    def update_config_by_key_val(self, ip: str, key: str, val: str):
        """update bench config by key and val

        Args:
            ip (str): the ip of bench
            key (str): such as "IO->FLM_L->phy_chans"
            val (str): the val of bench config to be updated

        """
        self.BENCH_DICT = Utils.update_bench_config_by_key_val(ip, key, val)
        self.__update_key_for_upper()

    def update_config_by_dict(self, newconfig):
        """
        Update devicemanager's current configuration with the user's  dict

        Args:
            newconfig (dict): user's configuration dict

        """
        self.BENCH_DICT = Utils.update_bench_config_by_dict(self.bench_ip, newconfig)
        self.__update_key_for_upper()

    def update_config_by_local_file(self, ip: str, path: str):
        """
        Update devicemanager's current configuration with the local file

        Args:
            ip (str): bench'ip
            path (str): user's configuration file path
        """
        self.BENCH_DICT = Utils.update_bench_config(ip, path)
        self.__update_key_for_upper()

    def get_can_configs(self):
        """Get CAN config info

        :return: dict of CAN config info.
        """
        return self.BENCH_DICT.get("CAN", None)

    def get_lin_configs(self):
        """Get LIN config info

        :return: dict of LIN config info.
        """
        return self.BENCH_DICT.get("LIN", None)

    def get_uds_configs(self):
        """Get the uds info.

        :return: uds info.
        """
        return self.BENCH_DICT.get("UDS", None)

    def get_io_configs(self, signal_name: str):
        """Return the channel and type of IO device.

        :param str signal_name: specifies signal name.
        :rtype: str,str
        :return: the channel and type of IO device.
        """
        if signal_name not in self.BENCH_DICT["IO"].keys():
            logger.error(f"{signal_name} not in bench-config file, please check")
            return
        io_dict = self.BENCH_DICT["IO"][signal_name]
        return io_dict["phy_chans"], io_dict["io_type"], io_dict["KWARGS"]

    def get_doip_ecu_ipaddr(self, ecu_name):
        """Get ecu ipaddr by ecu_name

        :return: str  ipaddress of special ecu_name .
        """
        VEHICLE_UNDER_TEST = os.getenv("VEHICLE_UNDER_TEST") or "Leo"
        logger.info(f"now vehicle type is {VEHICLE_UNDER_TEST}")
        if ecu_name not in self.BENCH_DICT["UDS"]["doip"][VEHICLE_UNDER_TEST].keys():
            logger.error(f"{ecu_name} not in uds.json, please check")
        ecu_doip_dict = self.BENCH_DICT["UDS"]["doip"][VEHICLE_UNDER_TEST][ecu_name]
        return ecu_doip_dict["ip_addr"]

    def get_doip_ecu_macaddr(self, ecu_name):
        """Get ecu macaddr by ecu_name

        :return: str  mac address of special ecu_name .
        """
        VEHICLE_UNDER_TEST = os.getenv("VEHICLE_UNDER_TEST") or "Leo"
        logger.info(f"now vehicle type is {VEHICLE_UNDER_TEST}")

        if ecu_name not in self.BENCH_DICT["UDS"]["doip"][VEHICLE_UNDER_TEST].keys():
            logger.error(f"{ecu_name} not in uds.json, please check")
            return

        ecu_doip_dict = self.BENCH_DICT["UDS"]["doip"][VEHICLE_UNDER_TEST][ecu_name]
        return ecu_doip_dict["mac_addr"]

    def get_ecu_logicaladdr(self, ecu_name, trans_type="doip"):
        """Get ecu logicaladdr by ecu_name

        :return: str logical address of special ecu_name .
        """

        VEHICLE_UNDER_TEST = os.getenv("VEHICLE_UNDER_TEST") or "Leo"
        logger.info(f"now vehicle type is {VEHICLE_UNDER_TEST}")
        if ecu_name not in self.BENCH_DICT["UDS"][trans_type][VEHICLE_UNDER_TEST].keys():
            logger.error(f"{ecu_name} not in uds.json, please check")
        ecu_doip_dict = self.BENCH_DICT["UDS"][trans_type][VEHICLE_UNDER_TEST][ecu_name]
        return ecu_doip_dict["logical_addr"]

    def parse_bus_name(self, bus_name: str) -> Tuple[Optional[str], str]:
        """Raw string should match speficfic pattern

        :param str raw_name: raw name of specifiesed CAN/LIN BUS.
        :rtype: Tuple[str]
        :return: Tuple of bus_type and bus_name.
        """
        bus_type: Optional[str] = None
        for channel_name in self.BENCH_DICT.get("CAN"):
            if bus_name.upper() in channel_name.upper():
                bus_type = "CANFD" if "CANFD" in channel_name.upper() else "CAN"
        for channel_name in self.BENCH_DICT.get("LIN"):
            if bus_name.upper() in channel_name.upper():
                bus_type = "LIN"
        if not bus_type:
            logger.error("%s is invalid CAN or LIN name.", bus_name)
            raise
        return (bus_type, bus_name)

    def get_bus_chann(self, bus_name: str) -> str:
        """Return the bus physical channel name from bench."""
        # load configuration file and parse.
        bus_type, _ = self.parse_bus_name(bus_name)
        bus_key = "CAN" if bus_type == "CANFD" else bus_type
        for bus_channel_key in self.BENCH_DICT[bus_key]:
            if bus_name.upper() == bus_channel_key.upper() or bus_name.upper() == "_".join(bus_channel_key.upper().split("_")[:-1]):
                bus_name_key = bus_channel_key
                break
        bus_channel = self.BENCH_DICT[bus_key][bus_name_key]
        logger.info(f"channel of bus_name is {bus_channel}.")
        return bus_channel

    def get_bus_list(self, bus_type: str = None) -> List[str]:
        """Get bus name string list.

        :param str bus_type: specifies the bus type, which can only be "CAN or LIN",
            by default the value is None, which means get all kind of CAN LIN Buses.
        :rtype: List[str]
        :return: list of all bus name string.
        """
        # add input checking here
        if not bus_type:
            return []

        bus_type = bus_type.upper()
        return self.BENCH_DICT.get(bus_type)  # .keys()

    def get_ecu_list(self, ecu_type: str = None):
        """Get all real ECU on bench.

        :param str ecu_type: specifies which kind of ECU you want to get. Valid options are "HW", "SIM"
        :rtype: List[str]
        :return: list of ECU names.
        """
        ret: List = []
        ecus_dict = {}
        ecu_type = ecu_type or "HW"

        if ecu_type not in ("HW", "SIM"):
            logger.error(f"Unkown ECU type {ecu_type}, only 'HW' and 'SIM' is valid option.")
        else:
            ecus_dict = self.BENCH_DICT.get("ECUS", {})
            if ecus_dict:
                ret = ecus_dict.get(ecu_type, None)

            if not ret:
                logger.info(f"Cann't find {ecu_type} ecus in this bench.")
                ret = []

            logger.info(f"Get {ecu_type} list [{ret}]")

        return list(ret)

    @staticmethod
    def idl(save_path: str, soa_services: list) -> list:
        """Get idl files for test

        :return: list ,fail idl file list .
        """
        return Utils.get_idls(save_path, soa_services)

    @staticmethod
    def dbc(save_path: str, dbc_services: list) -> list:
        """Get dbc files for test

        :return: list ,fail dbc file list .
        """
        pass

    @staticmethod
    def ldf(save_path: str, ldf_services: list) -> list:
        """Get ldf files for test

        :return: list ,fail ldf file list .
        """
        pass


# if __name__ == "__main__":
#     os.environ["Monik_Zone_FL_CAN1"] = "vcan_obd"
#     os.environ["Monik_Zone_FL_CAN2"] = "vcan_backup"
#     dm = DeviceManager("110.110.110.110")
#     print(dm)
