"""
Author: chong.ma
Date: 2023-11-24 20:09:41
LastEditTime: 2024-02-28 05:57:03
FilePath: /root/bench_tool/bench_tool/utils/IOManagers.py
Description:

"""

import pprint as pp
import re
import subprocess
import time

import nidaqmx
import nidaqmx.system
from nidaqmx import constants
from nidaqmx.constants import *
from nidaqmx.constants import AcquisitionType  # type: ignore

from .Logger import get_logger

logger = get_logger("IOManagers")
MAX_RATE = 10000


class NI_Managers:
    def __init__(self):
        self._task = nidaqmx.Task()
        self._collections = []

    def __enter__(self, **kwargs):
        print("Entering the context")
        return self  # 返回一个用于上下文的对象

    def __exit__(self, type, value, traceback):
        print("Exiting the context")
        if self._task is not None:
            self._task.close()
            self._task = None

    def __del__(self):
        if self._task is not None:
            self._task.close()

    @property
    def devices(self):
        _system = nidaqmx.system.System.local()
        return _system.devices

    @property
    def timing(self):
        return self._task.timing

    @property
    def triggers(self):
        return self._task.triggers

    @property
    def collections(self):
        return self._collections

    @collections.setter
    def collections(self, data):
        return self._collections == data

    def __setup(self, io_type, phy_chans, **kwargs):
        _io_type = io_type
        _phy_chans = phy_chans if isinstance(phy_chans, list) else [phy_chans]

        if not self.__is_valid_phy_chan(_io_type, _phy_chans):
            pp.pprint(f"Physical Channel: {phy_chans} not found with io type {io_type}.\n------------------\n", width=1000)
            pp.pprint(f"available Channel: {self.__get_phy_chans(io_type)}", width=30)
            raise BaseException()

        # setup chans
        chans_attr = self.__get_func_from_obj(self._task, f"{_io_type}_channels")  # eg. self._task.di_channels
        for phy_chan in _phy_chans:
            self.__get_func_from_obj(
                chans_attr,
                _io_type,
                kwargs.get("io_subtype"),
                kwargs.get("pulse"),
            )(phy_chan, **kwargs["chan_params"])

        # setup timing
        if kwargs["timing_type"] and kwargs.get("timing_params"):
            timing_func = self.__get_timing_func(kwargs["timing_type"])
            rate = kwargs["timing_params"].get("rate", MAX_RATE / len(_phy_chans))

            if kwargs["register_event"]:
                kwargs["timing_params"]["sample_mode"] = AcquisitionType.CONTINUOUS

            if kwargs["timing_type"] in ("sample_clock", "pipelined_sample_clock"):
                timing_func(rate=rate, **kwargs["timing_params"])
            else:
                timing_func(**kwargs["timing_params"])
            # setup event callback
            if kwargs["register_event"]:
                self.__add_callback()

    def __get_phy_chans(self, io_type):
        if "d" in io_type:
            attr = f"{io_type}_lines"
        else:
            attr = f"{io_type}_physical_chans"
        chans = [chan for device in self.devices.device_names for chan in getattr(nidaqmx.system.Device(device), attr).channel_names]
        # logger.info(f"physical channels: {chans}")
        return chans

    def __get_func_from_obj(self, obj, io_type, io_subtype="", pulse=False):
        try:
            return getattr(obj, io_type)
        except AttributeError:
            if io_subtype:
                if not pulse:
                    return getattr(obj, f"add_{io_type}_{io_subtype}_chan")
                else:
                    return getattr(obj, f"add_{io_type}_pulse_chan_{io_subtype}")
            else:
                return getattr(obj, f"add_{io_type}_chan")

    def __is_valid_phy_chan(self, io_type, phy_chans):
        """check phy_chans

        Args:
            io_type (str): io type between `ai, ao, di, do, ci, co`.
            phy_chans (List[str]): physical channels of the deivce.

        Returns:
            bool: if all channel in phy_chans is exists
        """
        return set(phy_chans).issubset(set(self.__get_phy_chans(io_type)))

    def __add_callback(self, count=1000):
        """_summary_

        Args:
            count (int, optional): _description_. Defaults to 1000.
        """

        def callback(task_handle, every_n_samples_event_type, number_of_samples, callback_data):
            self.collections.extend(self._task.read(number_of_samples_per_channel=count))
            return 0

        self._task.register_every_n_samples_acquired_into_buffer_event(count, callback)

    def __get_timing_func(self, timing_type):
        timing_type_map = dict(
            sample_clock=self._task.timing.cfg_samp_clk_timing,
            handshake=self._task.timing.cfg_handshaking_timing,
            implicit=self._task.timing.cfg_implicit_timing,
            change_detection=self._task.timing.cfg_change_detection_timing,
            pipelined_sample_clock=self._task.timing.cfg_pipelined_samp_clk_timing,
        )
        return timing_type_map[timing_type]

    def __start(self):
        self._task.start()

    def __stop(self, wait_util_done=False):
        if wait_util_done:
            self._task.wait_until_done()
        self._task.stop()

    def __read(self, number_of_samples_per_channel=1, timeout=10.0):
        return self._task.read(number_of_samples_per_channel=number_of_samples_per_channel, timeout=timeout)

    def __write(self, data, auto_start=False, timeout=10.0):
        return self._task.write(data, auto_start=auto_start, timeout=timeout)

    def check_connection(self, device_ips):
        for cdaq, ip in device_ips.items():
            result = subprocess.run(["ping", "-c", "1", ip], stdout=subprocess.PIPE, stderr=subprocess.PIPE)
            assert result == 0, f"Error checking {cdaq} {ip}"

    def di_read(self, phy_chans, **kwargs):
        """read di status

        Args:
            phy_chans (List[str]): physical channels of the deivce.
            **kwargs: timing_type, timing_params, register_event
        """
        self.__setup(
            io_type="di",
            phy_chans=phy_chans,
            io_subtype=kwargs.get("io_subtype", ""),
            chan_params=kwargs.get("chan_params", {}),
            timing_type=kwargs.get("timing_type", "sample_clock"),
            timing_params=kwargs.get("timing_params", None),
            register_event=kwargs.get("register_event", False),
        )
        return self.__read(1)

    def do_write(self, phy_chans, data, **kwargs):
        """write do status

        Args:
            phy_chans (List[str]): physical channels of the deivce.
            data (List[bool]): data to write. True or False
            **kwargs: timing_type, timing_params, register_event
        """
        self.__setup(
            io_type="do",
            phy_chans=phy_chans,
            io_subtype=kwargs.get("io_subtype", ""),
            chan_params=kwargs.get("chan_params", {}),
            timing_type=kwargs.get("timing_type", "sample_clock"),
            timing_params=kwargs.get("timing_params", None),
            register_event=kwargs.get("register_event", False),
        )
        return self.__write(data, auto_start=True)

    def ai_read(self, phy_chans, **kwargs):
        """read ai status"""
        self.__setup(
            io_type="ai",
            phy_chans=phy_chans,
            io_subtype=kwargs.get("io_subtype", "voltage"),
            chan_params=kwargs.get("chan_params", {}),
            timing_type=kwargs.get("timing_type", "sample_clock"),
            timing_params=kwargs.get("timing_params", None),
            register_event=kwargs.get("register_event", False),
        )
        return self.__read(5)

    def ao_write(self, phy_chans, data, **kwargs):
        """write ao status

        Args:
            phy_chans (_type_): _description_
            data (_type_): eg 10.0

        Returns:
            _type_: _description_
        """
        self.__setup(
            io_type="ao",
            phy_chans=phy_chans,
            io_subtype=kwargs.get("io_subtype", "voltage"),
            chan_params=kwargs.get("chan_params", {}),
            timing_type=kwargs.get("timing_type", "sample_clock"),
            timing_params=kwargs.get("timing_params", None),
            register_event=kwargs.get("register_event", False),
        )
        return self.__write(data, auto_start=True)

    def ci_read(self, phy_chans, **kwargs):
        """read ci status"""
        if kwargs.get("io_subtype") == "freq":
            kwargs.setdefault("chan_params", {"meas_method": constants.CounterFrequencyMethod.DYNAMIC_AVERAGING})

        self.__setup(
            io_type="ci",
            phy_chans=phy_chans,
            io_subtype=kwargs.get("io_subtype", "duty_cycle"),
            chan_params=kwargs.get("chan_params", {}),
            timing_type=kwargs.get("timing_type", None),
            timing_params=kwargs.get("timing_params", None),
            register_event=kwargs.get("register_event", False),
            pulse=kwargs.get("pulse", False),
        )
        self._task.channels.ci_thresh_voltage = 4
        print(self._task.channels.ci_thresh_voltage)

        return self.__read(1)

    def co_write(self, phy_chans, freq, duty_cycle, **kwargs):
        """set co status"""
        self.__setup(
            io_type="co",
            phy_chans=phy_chans,
            io_subtype="freq",
            chan_params=kwargs.get("chan_params", {"freq": freq, "duty_cycle": duty_cycle}),
            timing_type=kwargs.get("timing_type", "implicit"),
            timing_params=kwargs.get("timing_params", {"sample_mode": constants.AcquisitionType.CONTINUOUS}),
            register_event=kwargs.get("register_event", False),
            pulse=kwargs.get("pulse", True),
        )
        print(self._task.channels.co_pulse_term)
        # self.__write(CtrFreq(freq=freq, duty_cycle=duty_cycle))
        self.__start()
        time.sleep(60)
        self.__stop()

    def add_reserve_devices(self, device_ips: dict):
        if subprocess.check_output("nilsdev", shell=True).decode("utf-8") != "":
            out = subprocess.check_output("nidaqmxconfig --find-net-devs", shell=True).decode("utf-8")
            for dev in out.split("\n")[:-1]:
                dev = dev.split(".")[0]
                try:
                    out = subprocess.check_output(f"nidaqmxconfig --unreserve {dev}", shell=True).decode("utf-8")
                    assert "Success" in out, f"unreserve {dev} failed"
                except Exception as e:
                    print(e)
                try:
                    out = subprocess.check_output(f"nidaqmxconfig --del-net-dev {dev}", shell=True).decode("utf-8")
                    assert "Success" in out, f"del-net-dev {dev} failed"
                    print(f"{dev} deleted.")
                except Exception as e:
                    print(e)

        for cdaq, ip in device_ips.items():
            try:
                out = subprocess.check_output(f"nidaqmxconfig --add-net-dev {ip} --reserve --name {cdaq}", shell=True).decode("utf-8")
                assert "Success" in out, f"add {ip} failed"
                device_name = re.search(r"Added: (.+)", out).group(1)
                out = subprocess.check_output(f"nidaqmxconfig --reserve {device_name} --force", shell=True).decode("utf-8")
                assert "Success" in out, f"reserve {device_name} failed"
                print(f"{cdaq} Added.")
            except Exception as e:
                print(e)

    def reserve_devices(self, device_ips):
        for ip in device_ips:
            nidaqmx.system.Device.add_network_device(ip)
        system = nidaqmx.system.System.local()
        for device in system.devices:
            device.reserve_network_device(override_reservation=True)
            print(device)

    def find_test_devices(self, daq_configs=None):
        list_unavailable = []
        list_unhealthy = []
        list_healthy = []
        list_in_use = []
        for device in self.devices:
            if device.product_type == "cDAQ-9189":  # 检查设备是否是 cDAQ9189 基座
                print(f"Device Name: {device.name} Type: {device.product_type} ip: {device.tcpip_ethernet_ip}")
                for module in device.chassis_module_devices:
                    print(f"Module: {module.name} Type: {module.product_type} slot: {module.compact_daq_slot_num}")
                print("\n")

        out = subprocess.run("nilsdev", shell=True, stdout=subprocess.PIPE, stderr=subprocess.PIPE, text=True)
        for dev in out.stdout.split("\n")[:-1]:
            try:
                nidaqmx.system.device.Device(dev).self_test_device()
                # assert "Success" in out, f"{dev} failed"
                list_healthy.append(dev.strip())
                print(dev + " is OK")
            except Exception as e:
                if e.error_code == -88705 or "Initializing" in dev or "Not Present" in dev:
                    list_unavailable.append(dev.strip())
                    print(dev + " is unavailable")
                elif e.error_code == -201105:
                    list_healthy.append(dev.strip())
                    list_in_use.append(dev.strip())
                    print(dev + "is in-use")
                else:
                    list_unhealthy.append(dev.strip())

        return list_healthy, list_unavailable, list_in_use, list_unhealthy
