'''
Author: chong.ma
Date: 2023-11-28 11:54:40
LastEditTime: 2024-01-19 20:03:36
FilePath: /root/bench_tool/bench_tool/utils/PWMManagers.py
Description: 
    用于控制PWM输入输出设备
    参考https://nio.feishu.cn/wiki/wikcn99lei6xhvnEkJOBrzbQBC4

'''

import os, sys

import serial

sys.path.append(os.path.dirname(os.path.dirname(__file__)))
from .Logger import get_logger
from .Utils import Utils 
logger = get_logger("PWMManager")


class PWMManager:

    # __instance = None

    # def __new__(cls, timeout=30):
    #     if not cls.__instance:
    #         cls.__instance = object.__new__(cls)
    #     return cls.__instance

    def __init__(self, signal, serial_port, address, channel, direction):
        # serial_port, channel = ("/dev/ttyUSB13", "4")

        if not os.path.exists(serial_port):
            raise ValueError(f"serial port '{serial_port}' does not exist")

        # 打开串口
        self.ser = serial.Serial(serial_port, 9600, timeout=0.5)
        self.channel = channel

    def __del__(self):
        pass

    def __enter__(self, **kwargs):
        print("Entering the context")
        return self  # 返回一个用于上下文的对象

    def __exit__(self, exc_type, exc_value, traceback):
        print("Exiting the context")
        if hasattr(self, "ser") and self.ser.is_open:
            self.ser.close()
            print("Serial port closed")

    def read_pwm(self):
        """set pwm output of channel 0~3
            CH0+ ~ CH3+：PWM输入正
            CH4+ ~ CH7+：PWM输出正
            CH0-~ CH7-：输出负（实际和电源地共地）

        Returns:
            int: freq (0~65535)
            float: duty-cycle(0~100.00)
        """
        channel = int(self.channel)

        if not (0x00 <= channel <= 0x03):
            raise ValueError("Channel must be between 0x0~0x3")

        # 构造读取命令
        command = bytearray.fromhex("01 03 00 8{:d} 00 02".format(channel))
        command += self.__crc16(command)
        # 发送命令并读取返回数据
        self.ser.write(command)
        response = self.ser.read(7)
        logger.info("read_pwm response: {0}".format(response))
        # 解析返回数据
        if len(response) == 7:
            frequency = int.from_bytes(response[3:5], byteorder="big")
            duty_cycle = round((int.from_bytes(response[5:7], byteorder="big")) / 100, 2)
            return frequency, duty_cycle
        else:
            return (-1, -1)

    def set_pwm(self, frequency: int, duty_cycle: float):
        """set pwm output of channel 4~7
            CH0+ ~ CH3+：PWM输入正
            CH4+ ~ CH7+：PWM输出正
            CH0- ~ CH7-：输出负（实际和电源地共地）
        Args:
            frequency (int): freq of pwm, (0~65535Hz)
            duty_cycle (float): dutycycle of pwm, (0~100.00)
        """

        channel = int(self.channel, 16)

        if not (0x4 <= channel <= 0x7):
            raise ValueError("Channel must be between 0x4~0x7")

        if not isinstance(frequency, int):
            raise ValueError("Frequency must be an integer")

        if not (0 <= frequency <= 65535):
            raise ValueError("Frequency must be between 0 and 65535")

        if not isinstance(duty_cycle, (int, float)):
            raise ValueError("Duty cycle must be a numeric value")

        if not (0 <= duty_cycle <= 100.0):
            raise ValueError("Duty cycle must be between 0 and 100.00")

        # 构造设置频率的命令
        rounded_frequency = round(frequency)
        # freq_command = bytearray.fromhex('01 06 08 40 00 0A E9')
        freq_command = bytearray.fromhex("01 06 08 {:02X} {:04X}".format(0x40 + (channel - 4) * 2, rounded_frequency))
        freq_command += self.__crc16(freq_command)
        # 发送设置频率的命令
        self.ser.write(freq_command)
        response = self.ser.read(8)
        assert len(response) >= 8, "freq_command send error."
        logger.info("freq_command response: {0}".format(response))

        # 构造设置占空比的命令
        duty = hex(round(duty_cycle * 100))[2:].zfill(4)
        duty_command = bytearray.fromhex("01 06 08 {:02X} {:s} {:s}".format(0x41 + (channel - 4) * 2, duty[:2], duty[2:]))
        # duty_command = bytearray.fromhex('01 06 08 41 04 74 0C')
        duty_command += self.__crc16(duty_command)
        # 发送设置占空比的命令
        self.ser.write(duty_command)
        response = self.ser.read(8)
        assert len(response) >= 8, "duty_command send error."
        logger.info("duty_command response: {0}".format(response))

    def __crc16(self, data: bytes) -> bytes:
        crc = 0xFFFF
        for byte in data:
            crc ^= byte
            for _ in range(8):
                if crc & 0x0001:
                    crc = (crc >> 1) ^ 0xA001
                else:
                    crc >>= 1
        return crc.to_bytes(2, "little")

