"""
Author: chong.ma
Date: 2024-03-15 22:36:46
LastEditTime: 2024-03-26 13:37:37
FilePath: /root/bench_tool/bench_tool/utils/PowerManagers.py
Description: https://www.kikusui.cn/resources/001/pdf/pwr-01_users_manual.pdf
https://pyvisa.readthedocs.io/en/latest/index.html

"""

import os
import pprint as pp
import sys
import time

import pyvisa as visa
import serial
from pyvisa.constants import *
from pyvisa.resources.tcpip import TCPIPInstrument, TCPIPSocket
from pyvisa.resources.usb import USBInstrument

sys.path.append(os.path.dirname(os.path.dirname(__file__)))
from .Logger import get_logger
from .Utils import Utils

logger = get_logger("PowerManager")

CMDS = [
    # 488.2公共命令
    "*IDN?\n\r",
    # kikusui SCPI命令
    "VOLT <NR2>\n",
    "CURR <NR2>\n",
    "OUTPUT <Bool>\n",
    "OUTPUT?\n",
    "SOUR:VOLTAGE?\n",
    "SOUR:CURRENT?\n",
    "MEAS:VOLT?\n",
    "MEAS:CURR?\n",  # 8
    # ngi SCPI命令
    "SOURce:VOLTage <NR2>\n",
    "SOURce:CURRent <NR2>\n",
    "OUTPut:ONOFF <Bool>\n",
    "OUTPut:ONOFF?\n",
    "OUTPut:MODe <CRD>\n",
    "OUTPut:MODe?\n",
    "OUTPut:PRIority <CRD>\n",
    "OUTPut:PRIority?\n",
    "OUTPut:EVENt?\n",
    "OUTPut:STATe?\n",
    "MEASure:VOLTage?\n",
    "MEASure:CURRent?\n",
    "MEASure:POWer?\n",
    "MEASure:VOLTage:MAXimum?\n",
    "MEASure:CURRent:MAXimum?\n",
    "MEASure:POWer:MAXimum?\n",
]


class VISAInstrument:
    """
    Make sure the ps is in visa  before using this class
    """

    def __init__(self, visa_device):
        self.rm = visa.ResourceManager("/usr/lib/x86_64-linux-gnu/libvisa.so.24.0.0")
        self.dev_name = visa_device
        self.ins = None

    def __enter__(self):
        self.ins = self.rm.open_resource(self.dev_name)
        print(self.ins)
        return self

    def __exit__(self, exc_type, exc_val, exc_tb):
        self.rm.close()

    def query(self, query_str, delay=1) -> str:
        self.ins.write(query_str)
        time.sleep(delay)
        return self.ins.read(termination="\n")

    def write(self, cmd):
        self.ins.write(cmd)

    def read(self, cmd):
        self.ins.read(cmd)

    def list_device(self) -> list:
        devices = self.rm.list_resources()
        pp.pprint(f"available devices: {devices}")
        return devices

    def exec_cmd(self, cmd, val=None) -> str:
        print(cmd)
        val = str(val)
        try:
            if "?" in cmd:
                return self.query(cmd)
            elif "<NR2>" in cmd:
                cmd = cmd.replace("<NR2>", val)
                self.write(cmd)
            elif "<Bool>" in cmd:
                cmd = cmd.replace("<Bool>", val)
                self.write(cmd)
            elif "<CRD>" in cmd:
                cmd = cmd.replace("<CRD>", val)
                self.write(cmd)
            else:
                raise Exception("command not found")
        except Exception as e:
            raise Exception("command exec failed with error: ", e)

    def set_voltage_value(self, volt: float, inst="kikusui"):
        cmd = CMDS[1] if inst == "kikusui" else CMDS[9]
        self.exec_cmd(cmd, volt)

    def set_current_value(self, current: float, inst="kikusui"):
        cmd = CMDS[2] if inst == "kikusui" else CMDS[10]
        self.exec_cmd(cmd, current)

    def set_output(self, enable: bool, inst="kikusui"):
        cmd = CMDS[3] if inst == "kikusui" else CMDS[11]
        self.exec_cmd(cmd, "ON" if enable else "OFF")

    def read_output(self, inst="kikusui") -> bool:
        cmd = CMDS[4] if inst == "kikusui" else CMDS[12]
        return True if "ON" in str(self.exec_cmd(cmd)) else False

    def read_actualVolt(self, inst="kikusui") -> float:
        cmd = CMDS[7] if inst == "kikusui" else CMDS[19]
        return float(self.exec_cmd(cmd))

    def read_actualCurrent(self, inst="kikusui") -> float:
        cmd = CMDS[8] if inst == "kikusui" else CMDS[20]
        return float(self.exec_cmd(cmd))

    def read_idn(self) -> float:
        print(self.exec_cmd(CMDS[0]))


# if __name__ == '__main__':
#     with VISAInstrument("TCPIP0::192.168.1.202::7000::SOCKET") as ins:
#     # with VISAInstrument("TCPIP0::192.168.1.201::5025::SOCKET") as ins:
#     # with VISAInstrument("TCPIP::192.168.1.201::hislip0,4880::INSTR") as ins:
#         # ins.read_idn()
#         print(ins.read_actualVolt("ngi"))
#         print(ins.read_actualCurrent("ngi"))
#         print(ins.read_output("ngi"))
#         # ins.set_current_value(10, "ngi")
#         # ins.set_voltage_value(12, "ngi")
#         ins.set_output(False, "ngi")
