"""
Author: chong.ma
Date: 2023-10-12 17:34:48
LastEditors: Please set LastEditors
FilePath: /root/bench_tool/bench_tool/utils/RelayManagers.py
Description:
用于控制网口继电器https://nio.feishu.cn/wiki/LLexwrr6xihrtNkia3Uc071pnth
用于控制串口继电器https://nio.feishu.cn/wiki/wikcnxGsVQmljBBWXbVf5bE12Bb
用于控制箱式继电器https://nio.feishu.cn/file/KNVgbjt1qoapBdx7G2pc5aHfnzb
Copyright (c) 2023 by chong.ma@nio.com, All Rights Reserved.
"""

import argparse
import os
import re
import sys
import time

import serial
from pymodbus.client import ModbusTcpClient
from serial.tools import list_ports

RED = "\033[1;31m"
GREEN = "\033[1;32m"
YELLOW = "\033[1;33m"
BLUE = "\033[1;34m"
RESET = "\033[0m"


class RelayBoardDriver:
    def __init__(self, ip="192.168.1.250"):
        self.ip_address = ip
        self.port = 1030
        self.unit_id = 0x01
        self.client = ModbusTcpClient(self.ip_address, self.port)

    def __enter__(self, **kwargs):
        print("Entering the context")
        # 创建Modbus TCP客户端
        self.client.connect()
        return self  # 返回一个用于上下文的对象

    def __exit__(self, exc_type, exc_value, traceback):
        if hasattr(self, "client") and self.client.is_active():
            self.client.close()
            print("Board closed")
        print("Exiting the context")

    # 计算CRC16校验值
    def __calculate_crc(self, data: bytes) -> bytes:
        crc = 0xFFFF
        for byte in data:
            crc ^= byte
            for _ in range(8):
                if crc & 0x0001:
                    crc = (crc >> 1) ^ 0xA001
                else:
                    crc >>= 1
        return crc.to_bytes(2, "little")

    def __send(self, modbus_frame: bytearray):
        # 计算CRC16校验值并添加到Modbus帧
        crc_value = self.__calculate_crc(modbus_frame)
        modbus_frame.extend(crc_value)
        print("Sending Modbus frame: ", modbus_frame)
        assert self.client.send(modbus_frame) == 8, "Modbus frame not sent"
        return True

    def deal_all(self, status, ch_start=0, ch_end=8):
        if status not in ["RESET", "ON", "OFF"]:
            return  # 处理无效状态
        values = {"RESET": [0x00, 0xFF], "ON": [0xFF, 0xFF], "OFF": [0x00, 0x00]}
        for act in range(2 if status == "RESET" else 1):
            for coil in range(ch_start, ch_end):
                self.__send(bytearray([self.unit_id, 0x05, 0x00, coil, values[status][act], 0x00]))
                time.sleep(0.01)

            if status == "RESET":
                time.sleep(3)

    def channel_ctrl(self, status, channel):
        if status == "RESET":
            self.__send(bytearray([self.unit_id, 0x05, 0x00, int(channel), 0x00, 0x00]))
            time.sleep(0.5)
            self.__send(bytearray([self.unit_id, 0x05, 0x00, int(channel), 0xFF, 0x00]))
            time.sleep(0.5)
            return True
        else:
            coil_address = int(channel)
            coil_status = 0xFF if status == "ON" else 0x00
            # 构造Modbus帧
            modbus_frame = bytearray(
                [
                    self.unit_id,  # 单元标识符
                    0x05,  # 功能码
                    0x00,  # 起始地址（高字节）
                    coil_address,  # 起始地址（低字节）
                    coil_status,  # 线圈状态（高字节）
                    0x00,  # 线圈状态（低字节）
                ]
            )
            self.__send(modbus_frame)
            time.sleep(0.5)
            return True


class RelayManager:
    def __init__(self, sig_name, ser_port, addr, channel, dir):
        if not os.path.exists(ser_port):
            print(f"{RED}serial port '{ser_port}' does not exist{RESET}")
            raise ValueError("Can't find serial port of relay")

        self.ser = serial.Serial(ser_port, 115200, bytesize=8, timeout=0.1)
        self.channel = channel
        self.direction = dir
        self.addr = "{:02d}".format(int(addr, 2))  # 继电器控制板Addr 000000~111111
        self.sig_name = sig_name

    def __del__(self):
        pass

    def __enter__(self, **kwargs):
        print("Entering the context")
        if hasattr(self, "ser") and not self.ser.is_open:
            self.ser.open()
            print("Serial port open")
        return self  # 返回一个用于上下文的对象

    def __exit__(self, exc_type, exc_value, traceback):
        if hasattr(self, "ser") and self.ser.is_open:
            self.ser.close()
            print("Serial port closed")

    def __command_send(self, command, resp_code):
        """send command and get response

        Args:
            command (str): the command sending to relay controller
            resp_code (str): the response code corresponding to command

        Raises:
            ValueError: no response

        Returns:
            str: response of command
        """
        print(f"{GREEN}command '{command}'{RESET}")
        self.ser.write(bytes.fromhex(command))
        response = self.ser.readline().hex()
        print(f"{GREEN}response '{response}'{RESET}")

        assert response != "", f"{RED}response is None{RESET}"
        assert response.startswith("483a" + self.addr + resp_code), f"{RED}response error.{RESET}"
        return response

    def __multi_read_input(self):
        """read relay input status

        Returns:
            str: eg."483A00411010100000000000024544"
        """
        command = "48 3A " + self.addr + " 52 00 00 00 00 00 00 00 00 ff 45 44"
        check_sum = format(sum([int(pair, 16) for pair in command.split()][:12]) & 0xFF, "02x")
        command = "48 3A " + self.addr + " 52 00 00 00 00 00 00 00 00 " + check_sum + " 45 44"
        res = self.__command_send(command, "41")
        return res

    def __multi_set_relay(self, states):
        """set multi states of relay

        Args:
            states (str): 16 bits of relay states(1~16) eg."01ffff00ff0000ff"

        Returns:
            str: eg."483A00541010100000000000024544"
        """
        pairs = [states[i : i + 2] for i in range(0, len(states), 2)]
        swapped_pairs = [pair[1] + pair[0] for pair in pairs]
        swapped_pairs += ["ff"] * (8 - len(swapped_pairs))
        states = " ".join(swapped_pairs)
        command = "48 3A " + self.addr + " 57 " + states + " ff 45 44"
        check_sum = format(sum([int(pair, 16) for pair in command.split()][:12]) & 0xFF, "02x")
        command = "48 3A " + self.addr + " 57 " + states + check_sum + " 45 44"
        res = self.__command_send(command, "54")
        return res

    def __multi_read_relay(self):
        """read relay output status

        Returns:
            str: eg."483A00541010100000000000024544"
        """
        command = "48 3A " + self.addr + " 53 00 00 00 00 00 00 00 00 ff 45 44"
        check_sum = format(sum([int(pair, 16) for pair in command.split()][:12]) & 0xFF, "02x")
        command = "48 3A " + self.addr + " 53 00 00 00 00 00 00 00 00 " + check_sum + " 45 44"
        res = self.__command_send(command, "54")
        return res

    def __single_set_relay(self, state, channel):
        """set relay single output status

        Args:
            state (str): 01-On 00-Off
            channel(str): ch of relay 0x01~0x10
        Returns:
            str: eg."483A0071010100004544"
        """
        command = "48 3A " + self.addr + " 70 " + channel + " " + state + " 00 00 45 44"
        res = self.__command_send(command, "71")
        return res

    def __single_read_relay(self, channel):
        """read relay single output status

        Returns:
            _type_: _description_
        """
        command = "48 3A " + self.addr + " 72 " + channel + " ff 00 00 45 44"
        res = self.__command_send(command, "71")
        return res

    def set_relay_single(self, state):
        """set relay status of channel

        Args:
            state (str): On or Off or Reset
        """
        if state == "On":
            res = self.__single_set_relay("01", self.channel)
            assert res[8:12].upper() == self.channel + "01", f"{RED}Command failed{RESET}"
            print(f"{GREEN}Set relay digital output of channel {self.channel} is {state}{RESET}")
        elif state == "Off":
            res = self.__single_set_relay("00", self.channel)
            assert res[8:12].upper() == self.channel + "00", f"{RED}Command failed{RESET}"
            print(f"{GREEN}Set relay digital output of channel {self.channel} is {state}{RESET}")
        elif state == "Reset":
            res = self.__single_set_relay("00", self.channel)
            assert res[8:12].upper() == self.channel + "00", f"{RED}Command failed{RESET}"
            print(f"{GREEN}Set relay digital output of channel {self.channel} is {state}{RESET}")
            time.sleep(3)
            res = self.__single_set_relay("01", self.channel)
            assert res[8:12].upper() == self.channel + "01", f"{RED}Command failed{RESET}"
            print(f"{GREEN}Set relay digital output of channel {self.channel} is {state}{RESET}")

    def set_relay_double(self, status, clear_all=False):
        if clear_all is True:
            command = "0000000000000000"
        else:
            command = "ffffffffffffffff"

        chs = [(int(ch, 16) - 1) for ch in self.channel]
        if status == "LOAD":
            for idx, char in [(chs[0], "1"), (chs[1], "0")]:
                command = command[:idx] + char + command[idx + 1 :]
        elif status == "GND":
            for idx, char in [(chs[0], "0"), (chs[1], "1")]:
                command = command[:idx] + char + command[idx + 1 :]
        elif status == "NULL":
            for idx, char in [(chs[0], "0"), (chs[1], "0")]:
                command = command[:idx] + char + command[idx + 1 :]
        res = self.__multi_set_relay(command)
        pairs = [res[8:24][i : i + 2] for i in range(0, len(res[8:24]), 2)]
        swapped_pairs = [pair[1] + pair[0] for pair in pairs]
        result = "".join(swapped_pairs)

        # assert result == command.replace("f", "0"), f"{RED}Command failed{RESET}"
        print(f"{GREEN}Set relay digital output of signal {self.sig_name} is {result}{RESET}")

    def read_input(self):
        """get relay status of signal

        Returns:
            Bool: True-On False-Off
        """
        res = self.__multi_read_input()
        pairs = [res[8:24][i : i + 2] for i in range(0, len(res[8:24]), 2)]
        swapped_pairs = [pair[1] + pair[0] for pair in pairs]
        result = "".join(swapped_pairs)
        assert result[int(self.channel) - 1] in ("0", "1"), f"{RED}read_input get result error bit{RESET}"
        if result[int(self.channel) - 1] == "1":
            state = "On"
        else:
            state = "Off"
        print(f"{GREEN}Read relay digital input of signal {self.sig_name} is {RED}{state}{RESET}")
        return state == "On"


class BoxRelay:
    def __init__(self):
        pass

    def __del__(self):
        pass

    def __enter__(self, **kwargs):
        print("Entering the context")
        return self  # 返回一个用于上下文的对象

    def __exit__(self, exc_type, exc_value, traceback):
        print("Exiting the context")
        pass
