'''
Author: chong.ma
Date: 2023-10-23 13:32:36
LastEditTime: 2024-04-18 16:39:38
FilePath: /root/bench_tool/bench_tool/utils/ResManagers.py
Description: ctrl for res-board including set id/set channel res
Instruction papers: https://nio.feishu.cn/wiki/K8ktwGy0uilSXckubEZc00A8nAb?chunked=false

'''

import argparse
import re
import subprocess
import can
import time
class ResManager:
    def __init__(self, channel='can3', bitrate=500000):
        self.channel = channel
        self.res_can = can.interface.Bus(bustype='socketcan', channel=self.channel, bitrate=bitrate)    
        
    
    def __enter__(self):
        print("Entering the context")
        status = subprocess.getoutput(f"ifconfig {self.channel}")
        if "UP" not in status:
            print(f"you should set {self.channel} up")
            self.res_can.shutdown()
            exit(1)
        return self
    
    def __exit__(self, exc_type, exc_val, exc_tb):
        print("Exiting the context")
        self.res_can.shutdown()
        pass
    
    def __send(self, mesg):
        try:
            self.res_can.send(mesg)
            res = self.res_can.recv()
            assert res is not None, f"cmd {mesg} send failed"
            return res
        except:
            raise 
               
    def single_res_cmd(self, res_id, res_ch , res_val, res_sw = "On"):
        """set resistance box channel value

        Args:
            res_id (str): resistance box id, should be like 0001
            res_ch (str): channel of resistance box, should be like 1~16
            res_val (str): val of resistance box channel 12~20k ohm, res 5 ohm
            res_sw (str, optional): . Defaults to "On".
        """
        # 将通道信息转换为4位二进制字符串，其中第一位表示开关状态，后三位表示通道号
        channel = bin(int(res_ch) - 1)[2:].zfill(4)
        resistence = bin(int(res_val))[2:].zfill(32)  
        box_id = int("0x" + str(res_id), 16)    
        if res_sw  == "On":
            cmd = "1" + channel + resistence[5:]
        else:
            cmd = "0" + channel + resistence[5:]
        cmd_list = [cmd[i:i+4] for i in range(0, len(cmd), 4)]  
        print(f"cmd is {cmd_list}")        
        # 将二进制字符串转换为十六进制并组合
        hex_arr = [format(int(''.join(cmd_list[i:i+2]), 2), '02X') for i in range(0, len(cmd_list), 2)]
        # 反转列表，如果需要的话
        hex_arr = ["00", "00", "00", "00"] + hex_arr[::-1]
        hex_arr = bytes(int("0x" + x, 16) & 0xFF for x in hex_arr)  # Mask to ensure the values are within 0-255
        print(f"hex_arr is {hex_arr}")
        self.__send(can.Message(arbitration_id=box_id, data=hex_arr))
        print(f"set res box {res_id} where ch {res_ch} is {res_val} state {res_sw}")

    def double_res_cmd(self):
        # cover_data = ["00", "00", "00", "00"]
        # if self.channel % 2 == 0:
        #     hex_arr[:0] = cover_data[0:4][::-1]  # 通道号为偶数时需要填充前导数据
        # else:
        #     hex_arr[:0] = cover_data[0:4][::-1]  # 通道号为奇数时需要填充后继数据
        # hex_arr = ["0x" + i for i in hex_arr]
        # hex_arr = bytes(int(x, 16) & 0xFF for x in hex_arr)  # Mask to ensure the values are within 0-255
        # return hex_arr
        pass
        
        
    def get_id(self):
        """get id of R box, should only be used when there is only one R box connected to the PC

        Returns:
            _type_: _description_
        """
        remote_frame = can.Message(arbitration_id=0x00000000, is_remote_frame=True)
        res = self.__send(remote_frame) # 发送远程帧
        match = re.search(r'ID: ([0-9a-fA-F]+)', str(res)) # 接收CAN数据帧并匹配ID
        assert match is not None, "ID not found in the text"
        res_id = match.group(1)
        print("Extracted ID:", res_id)
        return res_id
    
    def set_id(self, new_id):
        """set id of R box, should only be used when there is only one R box connected to the PC

        Args:
            new_id (str): should be like 0001
        """
        res_id = int("0x" + self.get_id(), 16)
        hex_arr = bytes.fromhex(new_id + "000000000000")
        remote_frame = can.Message(arbitration_id=res_id, is_remote_frame=True)
        for _ in range(7):
            self.__send(remote_frame)
            time.sleep(0.5)
        self.__send(can.Message(arbitration_id=res_id, data=hex_arr))
        time.sleep(1)
        cur_id = self.get_id()
        assert "0000" + new_id == cur_id, f"set id failed, current id is {cur_id}"
        print(f"set id {new_id} success")
        