"""
Author: chong.ma
Date: 2023-11-09 17:12:18
LastEditTime: 2023-12-14 01:29:16
FilePath: /root/.config/rop-agent/drivers/bench_tool/bench_tool/utils/Utils.py
Description:

"""
import json
import os
import shutil
import subprocess
import sys
import zipfile
from pathlib import Path

import requests

sys.path.append(os.path.dirname(os.path.dirname(__file__)))
from .Logger import get_logger

BENCH_TOOL_DIR = Path(__file__).resolve().parents[1]
BENCH_MAP_PATH = BENCH_TOOL_DIR / "bench_config/bench_map.json"
BASE_DIR = BENCH_TOOL_DIR / "bench_config"
logger = get_logger("util")

RED = "\033[1;31m"
GREEN = "\033[1;32m"
YELLOW = "\033[1;33m"
BLUE = "\033[1;34m"
RESET = "\033[0m"


class Utils:
    def __init__(self):
        pass

    @classmethod
    def execute_command(self, command, shell=False, print_result=False):
        """执行shell命令或者shell脚本

        Args:
            command (_type_): _description_
        """
        try:
            with subprocess.Popen(
                command,
                stdout=subprocess.PIPE,
                stderr=subprocess.STDOUT,
                text=True,
                bufsize=1,
                universal_newlines=True,
                shell=shell,
                executable="/bin/bash",
            ) as process:
                if print_result:
                    for line in process.stdout:
                        print(line.strip())

                process.wait()  # 等待脚本执行完成
                if process.returncode == 0:
                    print(f"{GREEN}Command {command} execution successful.{RESET}")
                else:
                    raise subprocess.SubprocessError(f"{RED}Command {command} execution failed.{RESET}")
        except Exception as e:
            raise subprocess.SubprocessError(f"{RED}Error executing command: {e}{RESET}")

    @classmethod
    def execute_py_command(self, script_path, *kwargs):
        cmd = ["python3", script_path, *kwargs]
        try:
            subprocess.run(cmd, check=True)
        except subprocess.CalledProcessError as e:
            print(f"执行脚本时出现错误: {e}")
            sys.exit(1)
        except FileNotFoundError:
            print(f"未找到Python解释器或脚本文件: {script_path}")
            sys.exit(1)

        print(f"{GREEN}Command {cmd} execution successful.{RESET}")

    @classmethod
    def get_config_path_by_ip(self, ip: str, config_dir: str):
        """find config file path by ip in config_dir

        Args:
            ip (str): _description_
            config_dir (str): _description_

        Returns:
            config_path: return str of config file path
        """
        for dirpath, dirnames, filenames in os.walk(config_dir):
            for name in filenames:
                if ip in name:
                    logger.info(f"bench config file path :{dirpath}/{name}")
                    # logger.info(f"Got it:{dirpath}/{name}")
                    return f"{dirpath}/{name}"
        else:
            logger.error(f"Not found {ip} in {config_dir} ")
            return False

    @classmethod
    def update_dict_by_dict(self, obj_dict, src_dict):
        for key, src_value in src_dict.items():
            if isinstance(src_value, dict):
                # If the value is a dictionary, recursively update
                obj_value = obj_dict.get(key, {})
                if not isinstance(obj_value, dict):
                    obj_value = {}  # If the existing value is not a dictionary, create an empty one
                obj_dict[key] = self.update_dict_by_dict(obj_value, src_value)
            else:
                # If the value is not a dictionary, update directly
                obj_dict[key] = src_value

        return obj_dict

    @classmethod
    def get_bench_config_file(self, ip: str):
        """
        Get whole bench config by base.json and special json file of bench ip
        Args:
            ip (str): ip of bench

        Returns:
            dict: whole bench config info .
        """
        # print("self.bench_ip", self.bench_ip)
        bench_config = self.get_config_path_by_ip(ip, BASE_DIR)
        if not bench_config:
            default_config = {
                "CAN": {},
                "LIN": {},
                "UDS": {},
                "IO": {},
                "BENCH_TYPE": "SELF",
            }
            logger.warning("Not found config file and system provide a default config.")
            return default_config
        base_config = f"{os.path.dirname(bench_config)}/base.json"
        with open(base_config, "r") as base, open(bench_config, "r") as bench:
            bench_base_dict = dict(json.load(base))
            bench_dict = dict(json.load(bench))
            return self.update_dict_by_dict(bench_base_dict, bench_dict)

    @classmethod
    def update_bench_config_by_key_val(self, ip: str, key_path: str, val: str) -> dict:
        """update bench config by key and val

        Args:
            ip (str): the ip of bench
            key (str): such as "IO->FLM_L->phy_chans"
            val (str): the val of bench config to be updated
        Returns:
            dict: updated bench config
        """
        bench_dict = self.get_bench_config_file(ip)
        bench_type = bench_dict.get("BENCH_TYPE", "self").lower()
        config_path = self.get_config_path_by_ip(ip, BASE_DIR)
        if not config_path:
            config_path = BENCH_TOOL_DIR / f"bench_config/{bench_type}/{ip}.json"

        def update_nested_key(data_dict, key_path, new_value):
            keys = key_path.split("->")
            current_dict = data_dict
            for key in keys[:-1]:
                if key in current_dict and isinstance(current_dict[key], dict):
                    current_dict = current_dict[key]

            last_key = keys[-1]
            if last_key in current_dict:
                current_dict[last_key] = new_value
                return data_dict
            else:
                return None

        new_dict = update_nested_key(bench_dict, key_path, val)
        assert new_dict is not None, "Not found key in bench config!"
        with open(config_path, "w+", encoding="UTF-8") as f:
            json.dump(new_dict, f, indent=4, ensure_ascii=False)
        return new_dict

    @classmethod
    def update_bench_config_by_dict(self, ip: str, data: dict) -> dict:
        """update bench config by new data dict

        Args:
            ip (str): _description_
            data (dict):

        Returns:
            dict: _description_
        """
        if data is not None:
            bench_type = data.get("BENCH_TYPE", "self").lower()
            config_path = self.get_config_path_by_ip(ip, BASE_DIR)
            if not config_path:
                config_path = BENCH_TOOL_DIR / f"bench_config/{bench_type}/{ip}.json"

            bench_dict = self.get_bench_config_file(ip)
            new_dict = self.update_dict_by_dict(bench_dict, data)
            with open(config_path, "w+", encoding="UTF-8") as f:
                json.dump(new_dict, f, indent=4, ensure_ascii=False)
            return new_dict
        else:
            return {}

    @classmethod
    def update_bench_config(self, ip: str, path: str) -> dict:
        """update bench config by new cfg

        Args:
            ip (str): _description_
            path (str): _description_

        Returns:
            dict: _description_
        """
        if Path(path) is not None:
            new_cfg = dict(json.load(open(Path(path), "r")))
            bench_type = new_cfg.get("BENCH_TYPE", "self").lower()
            config_path = self.get_config_path_by_ip(ip, BASE_DIR)
            if not config_path:
                config_path = BENCH_TOOL_DIR / f"bench_config/{bench_type}/{ip}.json"

            bench_dict = self.get_bench_config_file(ip)
            new_dict = self.update_dict_by_dict(bench_dict, new_cfg)
            with open(config_path, "w+", encoding="UTF-8") as f:
                json.dump(new_dict, f, indent=4, ensure_ascii=False)
            return new_dict
        else:
            return {}

    @classmethod
    def get_nuc_ip(self) -> str:
        """get present Bench's NUC IP

        Returns:
            _type_: _description_
        """

        try:
            cmd = "ls /sys/class/net/ | grep -E '^wl'"
            process1 = subprocess.Popen(cmd, shell=True, stdout=subprocess.PIPE, stderr=subprocess.PIPE, text=True)
            interface = process1.stdout.readline().strip()
            cmd = f"ip addr show {interface} | grep -oP '(?<=inet\s)\d+\.\d+\.\d+\.\d+' | grep -m 1 ''"
            process2 = subprocess.Popen(cmd, shell=True, stdout=subprocess.PIPE, stderr=subprocess.PIPE, text=True)
            ip = process2.stdout.readline().strip()
            if ip is None or ip == "":
                logger.error("Not get present Bench's NUC IP")
                ip = "123.123.123.123"  # give a default ip for unittest
            else:
                logger.info("Present Bench's NUC IP is\t:\t" + ip)
            return ip
        finally:
            process1.stdout.close()
            process1.stderr.close()
            process1.wait()

            process2.stdout.close()
            process2.stderr.close()
            process2.wait()

    @classmethod
    def get_benchname(self) -> str:
        """get bench name from env param BENCH_NAME

        Returns:
            str: bench name
        """
        env_bench_name = os.getenv("BENCH_NAME")
        assert env_bench_name is not None, "env param BENCH_NAME is None!"
        return env_bench_name

    @classmethod
    def set_benchname(self, ip: str, name: str) -> str:
        """set bench name by ip and name and save to bench_map.json and env param BENCH_NAME

        Args:
            ip (str): bench ip
            name (str): bench name
        """
        if name:
            bench_map = Path(BENCH_MAP_PATH)
            with open(bench_map) as file:
                map_dict = json.load(file)
                assert ip in map_dict.keys(), f"ip:{ip} not in bench_map.json"
                map_dict.update({ip: name})

            logger.info(f"set name:{ip}:{name} and save path:{bench_map}")
            os.environ["BENCH_NAME"] = name
            with open(bench_map, "w", encoding="UTF-8") as f:
                json.dump(map_dict, f, indent=4, ensure_ascii=False)

    @classmethod
    def get_idls(self, save_path: str, soa_services: list) -> list:
        """Return idl file list.
        :save_path: str ,保存idl的文件路径，需要用户指定
        :soa_services: list ,指定要下载的soa的服务名称以及对应的版本 ,[{"service_name":"srv.dc.audio.abc","version":"v1.5"}]
        :return: list ,返回下载失败的服务名称
        """
        url = ""
        response = requests.get(url)
        with open("/tmp/" + "idl_compressed_file_202306281406327630.zip", "wb") as file:
            file.write(response.content)
        source_folder = "/tmp/idl_compressed_file_202306281406327630"
        # 解压zip文件
        zip_ref = zipfile.ZipFile("/tmp/" + "idl_compressed_file_202306281406327630.zip", "r")
        zip_ref.extractall("/tmp")
        zip_ref.close()

        file_names = os.listdir(source_folder)
        # 移动文件到指定的目录路径下
        for file_name in file_names:
            source_path = os.path.join(source_folder, file_name)
            destination_path = os.path.join(save_path, file_name)
            shutil.move(source_path, destination_path)

        # 判断解压后的文件是否在全部包含指定的服务名
        file_list = os.listdir(save_path)

        fail_idl = []
        for i in soa_services:
            if not any([i.get("service_name").lower() in item.lower() for item in file_list]):
                fail_idl.append(i.get("service_name"))
                logger.info(f"failed to download idl file :{i.get('service_name')}")
        return fail_idl

    @classmethod
    def get_dbcs(self):
        pass

    @classmethod
    def get_ldfs(self):
        pass

    if __name__ == "__main__":
        # t = get_idls(
        #     "/Users/wiliam.wang/Works",
        #     [
        #         {"service_name": "srv.dc.audio.abc", "version": "v1.5"},
        #         {"service_name": "srv.dc.audio.abce", "version": "v1.5"},
        #     ],
        # )
        # print(t)
        update_bench_config_by_key_val("10.110.182.200", "IO->FLM_L->phy_chans", "can0")
